@@ -57,20 +57,13 @@ fn generate_parse_fun_body(data: &Data) -> TokenStream {
                 } else {
                     unimplemented!();
                 }
-                quote_spanned! { variant.span() => match ts.parse() {
-                    Ok(x) => return Ok(Self::#name(x)),
-                    Err(e) => {
-                        if error.as_ref().map_or(true, |x| x.position() < e.position()) {
-                            error = Some(e);
-                        }
-                    }
+                quote_spanned! { variant.span() => if let Ok(x) = ts.parse() {
+                    return Ok(Self::#name(x));
                 }}
             });
             quote! {
-
-                let mut error: Option<crate::parse::Error> = None;
                 #( #arms )*
-                Err(error.take().expect("unreachable"))
+                Err(ts.take_last_error().expect("unreachable"))
             }
         }
         Data::Union(_) => unimplemented!(),
@@ -188,15 +181,11 @@ pub fn derive_format_trait(input: proc_macro::TokenStream) -> proc_macro::TokenS
     let generics = add_format_trait_bounds(input.generics);
     let (impl_generics, ty_generics, where_clause) = generics.split_for_impl();
     let format = generate_format_method_body(&input.data);
-    let should_be_packed = generate_should_be_packed_method_body(&input.data);
     let expanded = quote! {
         impl #impl_generics crate::format::Format for #name #ty_generics #where_clause {
-            fn format(&self, fmt: &mut crate::format::Formatter) -> crate::format::Result<()> {
+            fn format(&self, fmt: &mut crate::format::Formatter)  {
                 #format
             }
-            fn should_be_packed(&self) -> bool {
-                #should_be_packed
-            }
         }
     };
     proc_macro::TokenStream::from(expanded)
@@ -217,11 +206,10 @@ fn generate_format_method_body(data: &Data) -> TokenStream {
             Fields::Named(ref fields) => {
                 let format = fields.named.iter().map(|f| {
                     let name = &f.ident;
-                    quote_spanned! { f.span() => self.#name.format(fmt)? }
+                    quote_spanned! { f.span() => self.#name.format(fmt) }
                 });
                 quote! {
                     #(#format ;)*
-                    Ok(())
                 }
             }
             Fields::Unnamed(ref fields) => {
@@ -250,7 +238,35 @@ fn generate_format_method_body(data: &Data) -> TokenStream {
     }
 }
 
-fn generate_should_be_packed_method_body(data: &Data) -> TokenStream {
+#[proc_macro_derive(Element)]
+pub fn derive_element_trait(input: proc_macro::TokenStream) -> proc_macro::TokenStream {
+    let input = parse_macro_input!(input as DeriveInput);
+    let name = input.ident;
+    let generics = add_element_trait_bounds(input.generics);
+    let (impl_generics, ty_generics, where_clause) = generics.split_for_impl();
+    let is_packable = generate_is_packable_method_body(&input.data);
+    let expanded = quote! {
+        impl #impl_generics crate::items::components::Element for #name #ty_generics #where_clause {
+            fn is_packable(&self) -> bool {
+                #is_packable
+            }
+        }
+    };
+    proc_macro::TokenStream::from(expanded)
+}
+
+fn add_element_trait_bounds(mut generics: Generics) -> Generics {
+    for param in &mut generics.params {
+        if let GenericParam::Type(ref mut type_param) = *param {
+            type_param
+                .bounds
+                .push(parse_quote!(crate::items::components::Element));
+        }
+    }
+    generics
+}
+
+fn generate_is_packable_method_body(data: &Data) -> TokenStream {
     match *data {
         Data::Struct(ref data) => match data.fields {
             Fields::Named(_) => {
@@ -258,7 +274,7 @@ fn generate_should_be_packed_method_body(data: &Data) -> TokenStream {
             }
             Fields::Unnamed(ref fields) => {
                 assert_eq!(fields.unnamed.len(), 1);
-                quote! { self.0.should_be_packed() }
+                quote! { self.0.is_packable() }
             }
             Fields::Unit => unimplemented!(),
         },
@@ -270,7 +286,7 @@ fn generate_should_be_packed_method_body(data: &Data) -> TokenStream {
                 } else {
                     unimplemented!();
                 }
-                quote_spanned! { variant.span() => Self::#name(x) => x.should_be_packed(), }
+                quote_spanned! { variant.span() => Self::#name(x) => x.is_packable(), }
             });
             quote! {
                 match self {
